"""Build monitor"""

import curses
import logging
import os
import sys
import threading
#  from pprint import pprint
from time import perf_counter, sleep, time

from yojenkins.monitor.monitor import Monitor
from yojenkins.utility.utility import get_resource_path
from yojenkins.yo_jenkins.status import Color, StageStatus, Status

from . import monitor_utility as mu

# Getting the logger reference
logger = logging.getLogger()


class BuildMonitor(Monitor):
    """This class defines the BuildMonitor class and its function.

    The BuildMonitor class enables active build monitoring
    """

    def __init__(self, rest, auth, Build) -> None:
        """Object constructor method, called at object creation

        Args:
            Build: Build objects

        Returns:
            None
        """
        # Get attributes from super (parent) class
        super().__init__()

        self.rest = rest
        self.auth = auth
        self.build = Build

        # Build Info Thread
        self.build_info_data = {}
        self.build_info_thread_interval = 0.0
        self._build_info_thread_lock = threading.Lock()

        # Build Stages Thread
        self.build_stages_data = {}
        self.build_stages_thread_interval = 0.0
        self._build_stages_thread_lock = threading.Lock()

        # Aborting build flag
        self.build_abort = 0

        # Output build logs to console
        self.build_logs = False

        # Temporary message box on screen
        self.message_box_temp_duration = 1  # sec

        self.sound_directory = ""

    ###########################################################################
    #                         BUILD MONITOR
    ###########################################################################

    def __monitor_draw(self, scr, build_url: str, sound: bool = False) -> bool:
        """
        Draw a the BUILD MONITOR UI on the screen

        Args:
            scr       : Handle for curses terminal screen handle
            build_url : Direct URL to build
            sound     : Enable sound effects
        Returns:
            True if no error, else False
        """
        # Starting data collection threads
        self.server_status_thread_on()
        self.__build_info_thread_on(build_url=build_url)
        self.__build_stages_thread_on(build_url=build_url)

        # Setting up basic stuff for curses and load keys
        self.basic_screen_setup(halfdelay=True)
        ui_keys = mu.load_keys()

        # Sound effect related
        self.sound_directory = get_resource_path(os.path.join('resources', 'sound'))
        sound_notify_msg_time = 0
        sound_notify_msg_show = False
        sound_notify_msg_box_timing = False
        status_sound_last = ''

        # User key input (ASCII value)
        keystroke = 0

        # Debug stuff
        # loop_total_time = 0

        # Main Loop
        while True:
            start_time = perf_counter()

            # Clearing the screen at each loop iteration before constructing the frame
            scr.clear()

            ########################################################################################

            # Check user keyboard input
            if keystroke in ui_keys['QUIT']:
                self.quit += 1
            elif keystroke in ui_keys['ABORT']:
                self.build_abort += 1
            elif keystroke in ui_keys['RESUME']:
                self.quit = 0
                self.build_abort = 0
            elif keystroke in ui_keys['PAUSE']:
                self.paused = not self.paused
            elif keystroke in ui_keys['HELP']:
                self.help = not self.help
            elif keystroke in ui_keys['OPEN']:
                if build_url:
                    self.build.browser_open(build_url=build_url)
            elif keystroke in ui_keys['SOUND']:
                sound = not sound
                sound_notify_msg_show = True
            elif keystroke in ui_keys['LOGS']:
                self.build_logs = True

            ########################################################################################

            # Check terminal size
            term_height, term_width = scr.getmaxyx()
            self.check_terminal_size(scr)

            # Paint background
            mu.paint_background(scr, self.color['normal'])

            ########################################################################################

            # TOP HEADER
            y_row = 1

            if logger.level < 20:
                # mu.draw_text(scr, f"{loop_total_time:.4f}", y_row, center_x=True, color=self.color['grey-light'], decor=self.decor['bold'])
                # mu.draw_text(scr, str(curses.baudrate()), y_row, center_x=True, color=self.color['grey-light'], decor=self.decor['bold'])
                mu.draw_text(scr,
                             str(time() - sound_notify_msg_time) + ' ' + str(sound) + ' ' + str(sound_notify_msg_show),
                             y_row,
                             center_x=True,
                             color=self.color['grey-light'],
                             decor=self.decor['bold'])
            else:
                mu.draw_text(scr,
                             'BUILD MONITOR',
                             y_row,
                             center_x=True,
                             color=self.color['grey-light'],
                             decor=self.decor['bold'])
            y_row += 1

            # Draw header divider
            mu.draw_horizontal_seperator(scr, y_row, self.color['grey-dark'])
            y_row += 2

            ########################################################################################

            # INFO SECTION
            x_col = [3, 16]
            if self.build_info_data:
                # Get the build_url
                build_url = self.build_info_data['url']

                # INFO
                mu.draw_horizontal_header(scr, y_row, x_col[0], term_width - 5, '-', 'INFO',
                                          self.color['normal'] | self.decor['bold'])
                y_row += 1

                build_info_items = {
                    'Job': self.build_info_data['jobName'],
                    'Build': self.build_info_data['displayName'],
                    'Folder': self.build_info_data['folderFullName'],
                    'Server': self.build_info_data['serverURL'],
                    'Executor': self.build_info_data['builtOn']
                }
                for i, (key, value) in enumerate(build_info_items.items()):
                    # Job Name
                    mu.draw_text(scr, f'{key}:', y_row, x_col[0], decor=self.decor['bold'])
                    mu.draw_text(scr, mu.truncate_text(f'{value}', term_width - 5 - 12), y_row, x_col[1])
                    y_row += 1
                y_row += 1

                mu.draw_horizontal_header(scr, y_row, x_col[0], term_width - 5, '-', 'STATUS',
                                          self.color['normal'] | self.decor['bold'])
                y_row += 1

                # Play a sound on job status change
                if sound:
                    mu.draw_text(scr,
                                 '( fx )',
                                 1,
                                 term_width - 8,
                                 color=self.color['grey-dark'],
                                 decor=self.decor['bold'])
                if sound and not self.playing_sound:
                    # Get the sound file name
                    status_sound = self.status_to_sound(self.build_info_data['resultText'])
                    if status_sound_last != status_sound and status_sound:
                        # FIXME: Only check file names, not status text
                        self.play_sound_thread_on(os.path.join(self.sound_directory, status_sound))
                        status_sound_last = status_sound

                # Status text color
                status_color = self.status_to_color(self.build_info_data['resultText'])

                build_info_items = {
                    'Started': [self.build_info_data['startDatetime'], self.color['normal'], self.decor['normal']],
                    'Ended': [self.build_info_data['endDatetime'], self.color['normal'], self.decor['normal']],
                    'Elapsed': [self.build_info_data['elapsedFormatted'], self.color['normal'], self.decor['normal']],
                    'Estimated': [
                        self.build_info_data['estimatedDurationFormatted'], self.color['normal'], self.decor['normal']
                    ],
                    'Refresh': [
                        str(self.build_info_thread_interval) + ' sec', self.color['normal'], self.decor['normal']
                    ],
                    'Status': [self.build_info_data['resultText'], self.color[status_color], self.decor['bold']]
                }
                for i, (key, value) in enumerate(build_info_items.items()):
                    mu.draw_text(scr, f'{key}:', y_row, x_col[0], decor=self.decor['bold'])
                    mu.draw_text(scr,
                                 mu.truncate_text(f'{value[0]}', term_width - 5 - 12),
                                 y_row,
                                 x_col[1],
                                 color=value[1],
                                 decor=value[2])
                    y_row += 1
            else:
                y_row += 3
                mu.draw_text(scr,
                             'NO DATA',
                             y_row,
                             center_x=True,
                             color=self.color['normal'],
                             decor=self.decor['bold'])
                y_row += 2
                mu.draw_text(scr,
                             'ಠ_ಠ  ¯\_(⊙︿⊙)_/¯',
                             y_row,
                             center_x=True,
                             color=self.color['normal'],
                             decor=self.decor['bold'])
            y_row += 1

            ########################################################################################

            # TODO: Add QUEUED status for build status

            # STAGES SECTION
            if self.build_stages_data:
                x_col = [3, 8, 40, 55]

                # Header
                mu.draw_horizontal_header(scr, y_row, x_col[0], term_width - 5, '-', 'STAGES',
                                          self.color['normal'] | self.decor['bold'])
                y_row += 1

                # Loop through all listed stages in build
                for i, build_stage in enumerate(self.build_stages_data):
                    # Stage number
                    try:
                        mu.draw_text(scr, f'{i + 1}.', y_row, x_col[0])
                    except:
                        break

                    # Stage name
                    line = mu.truncate_text(build_stage['name'] if 'name' in build_stage else '-', 29)
                    mu.draw_text(scr, line, y_row, x_col[1])

                    # Stage Run duration
                    line = build_stage['durationFormatted'] if 'durationFormatted' in build_stage else '-'
                    mu.draw_text(scr, line, y_row, x_col[2])

                    # Status text and color
                    result_text = build_stage['status'] if 'status' in build_stage else StageStatus.UNKNOWN.value
                    status_color = self.status_to_color(build_stage['status'])

                    mu.draw_text(scr, result_text.replace("_", " "), y_row, x_col[3], color=self.color[status_color])
                    y_row += 1
            else:
                # Change the minimum window height limit (no stages section)
                self.height_limit = 17

            # Divider
            y_row = term_height - 4
            mu.draw_horizontal_seperator(scr, y_row, self.color['grey-dark'])

            ########################################################################################

            # SERVER STATUS
            y_row = term_height - 3
            if self.server_status_data:
                auth_status = False if 'auth' not in self.server_status_data else self.server_status_data["auth"]
                reach_status = False if 'reachable' not in self.server_status_data else self.server_status_data[
                    "reachable"]
                line = f'Server Status: Reachable: {reach_status}, Authenticated: {auth_status}'
            else:
                line = 'Server Status: NO DATA'
            mu.draw_text(scr, line, y_row, center_x=True, color=self.color['grey-dark'])

            ########################################################################################

            # User key input instructions
            y_row = term_height - 2
            mu.draw_text(scr, 'Press "H" for keyboard shortcuts', y_row, center_x=True, color=self.color['grey-dark'])

            ########################################################################################

            # Drawing the screen border
            border_color = 'grey-dark'
            if 'resultText' in self.build_info_data:
                if self.build_info_data['resultText'] in Status.SUCCESS.value:
                    border_color = Color.ITEMS.value['SUCCESS']
                elif self.build_info_data['resultText'] in Status.FAILURE.value:
                    border_color = Color.ITEMS.value['FAILURE']
                elif self.build_info_data['resultText'] in Status.ABORTED.value:
                    border_color = Color.ITEMS.value['ABORTED']
                elif self.build_info_data['resultText'] in Status.UNSTABLE.value:
                    border_color = Color.ITEMS.value['UNSTABLE']

            mu.draw_screen_border(scr, self.color[border_color])

            ########################################################################################

            # Indicate server interaction with icon
            if self.server_interaction:
                mu.draw_text(scr,
                             '(R)',
                             term_height - 2,
                             term_width - 5,
                             color=self.color['grey-dark'],
                             decor=self.decor['bold'])
            self.server_interaction = False

            ########################################################################################

            # Refresh the screen (scr.nourefresh + scr.doupdate)
            scr.refresh()

            ########################################################################################

            halfdelay_normal = False

            # Help message box
            if self.help:
                curses.halfdelay(255)
                message_lines = [
                    'A - Abort build', 'L - Output build logs', 'O - Open build in web browser', 'P - Pause Monitor',
                    'Q - Quit Monitor', 'S - Sound notification on/off', ' ', 'H - Keyboard shortcuts'
                ]
                mu.draw_message_box(scr, message_lines, 'left')
            else:
                halfdelay_normal = True

            # Sound effect notification on/off (Toggle)
            if sound_notify_msg_show:
                sound_notify_msg_show = False
                sound_notify_msg_time = time()
                sound_notify_msg_box_timing = True
            if sound_notify_msg_box_timing:
                if time() - sound_notify_msg_time < self.message_box_temp_duration:
                    state = 'ON' if sound else 'OFF'
                    mu.draw_message_box(scr, [f'Sound notification {state}'])
                else:
                    sound_notify_msg_box_timing = False

            # Pause message box
            if self.paused:
                self.help = False
                curses.halfdelay(255)
                message_lines = ['Monitor paused', 'Requests stopped', 'To resume press "P"']
                mu.draw_message_box(scr, message_lines)
            else:
                halfdelay_normal = True

            # Abort message box
            if self.build_abort:
                self.help = False
                curses.halfdelay(255)
                message_lines = ['Are you sure you want to abort build?', 'To abort press "A"', 'To return press "R"']
                mu.draw_message_box(scr, message_lines)
                if self.build_abort > 1:  # Abort Message confirmed (pressed twice)
                    if build_url:
                        self.server_interaction = True
                        build_number = self.build.abort(build_url=build_url)
                    else:
                        # TODO: Show UI error message for missing build_url
                        pass
                    self.build_abort = 0
            else:
                halfdelay_normal = True

            # Show the build logs
            if self.build_logs:
                self.help = False
                self.all_threads_enabled = False
                curses.echo(True)
                curses.nl(True)
                curses.endwin()
                if build_url:
                    self.server_interaction = True
                    self.build.logs(build_url=build_url)
                return True

            # Quit message box
            if self.quit:
                self.help = False
                curses.halfdelay(255)
                message_lines = ['Are you sure you want to quit?', 'To quit press "Q"', 'To return press "R"']
                mu.draw_message_box(scr, message_lines)
                # Quit Message confirmed (pressed twice)
                if self.quit > 1:
                    self.all_threads_enabled = False
                    return True
            else:
                halfdelay_normal = True

            # Screen refresh/updating to normal
            if halfdelay_normal:
                curses.halfdelay(self.halfdelay_screen_refresh)

            # Straight exist program
            if self.exit:
                self.all_threads_enabled = False
                sys.exit(0)

            ########################################################################################

            loop_total_time = perf_counter() - start_time

            # Get User input
            keystroke = scr.getch()

    def monitor_start(self, build_url: str, sound: bool = False) -> bool:
        """
        Curses wrapper function for drawing main menu on screen

        Args:
            build_url: Server URL of the build
            sound: If True, monitor is started with sound option on

        Returns:
            True, if successful, else False
        """
        # Disable any console output logging
        mu.logging_console(enabled=False)

        return curses.wrapper(self.__monitor_draw, build_url, sound)

    ###########################################################################
    #                      DATA COLLECTION THREADS
    ###########################################################################

    ###########################  BUILD INFO  ##################################

    def __thread_build_info(self, build_url: str, monitor_interval: float) -> None:
        """
        Independent thread which polls build information

        Args:
            build_url: Server URL of the build
            monitor_interval: Seconds between polling build URL

        Returns:
            None
        """
        logger.debug(
            f'Thread starting - Build info - (ID: {threading.get_ident()} - Refresh Interval: {monitor_interval}s) ...'
        )

        # Set the monitoring thread flag up
        self.all_threads_enabled = True
        self.build_info_thread_interval = monitor_interval

        # Loop until flags disable it
        while self.all_threads_enabled:
            if not self.paused:
                self.server_interaction = True
                with self._build_info_thread_lock:
                    self.build_info_data = self.build.info(build_url=build_url)

            # Wait some time before checking again
            start_time = time()
            while time() - start_time <= monitor_interval:
                sleep(0.100)
                if not self.all_threads_enabled:
                    break

        logger.debug(f'Thread stopped - Build info - (ID: {threading.get_ident()})')

    def __build_info_thread_on(self, build_url: str = '', monitor_interval: float = 7.0) -> bool:
        """
        Trigger to start independent thread that polls build information

        Args:
            build_url: Server URL of the build
            monitor_interval: Seconds between polling build URL

        Returns:
            True if successful, else False
        """
        logger.debug(f'Starting thread for build info for "{build_url}" ...')
        try:
            threading.Thread(target=self.__thread_build_info, args=(
                build_url,
                monitor_interval,
            ), daemon=False).start()
        except Exception as error:
            logger.error(
                f'Failed to start build info monitoring thread for {build_url}. Exception: {error}. Type: {type(error)}'
            )

        return True

    ###########################  BUILD STAGES  ################################

    def __thread_build_stages(self, build_url: str, monitor_interval: float) -> None:
        """
        Independent thread that polls build stages

        Args:
            build_url: Server URL of the build
            monitor_interval: Seconds between polling build URL

        Returns:
            None
        """
        logger.debug(
            f'Thread starting - Build Stages - (ID: {threading.get_ident()} - Refresh Interval: {monitor_interval}s) ...'
        )

        # Set the monitoring thread flag up
        self.all_threads_enabled = True
        self.build_stages_thread_interval = monitor_interval

        # Check if this is a staged build
        logger.debug(f'Checking if build is a staged build ...')
        request_url = f"{build_url.strip('/')}/wfapi/describe"
        return_content, _, return_success = self.rest.request(request_url, 'get', is_endpoint=False)
        if not return_success or not return_content:
            logger.debug('Failed to get build stages. This may not be a staged build')
            return

        # Loop until flags disable it
        while self.all_threads_enabled:
            if not self.paused:
                self.server_interaction = True
                with self._build_stages_thread_lock:
                    self.build_stages_data = self.build.stage_list(build_url=build_url)[0]

            # Wait some time before checking again
            start_time = time()
            while time() - start_time <= monitor_interval:
                sleep(0.1)
                if not self.all_threads_enabled:
                    break

        logger.debug(f'Thread stopped - Build Stages - (ID: {threading.get_ident()})')

    def __build_stages_thread_on(self, build_url: str = '', monitor_interval: float = 9.0) -> bool:
        """
        Trigger to start independent thread that polls build stages

        Args:
            build_url: Server URL of the build
            monitor_interval: Seconds between polling build URL

        Returns:
            True if successful, else False
        """
        logger.debug(f'Starting thread for build stages for "{build_url}" ...')
        try:
            threading.Thread(target=self.__thread_build_stages, args=(
                build_url,
                monitor_interval,
            ), daemon=False).start()
        except Exception as error:
            logger.error(
                f'Failed to start build info monitoring thread for {build_url}. Exception: {error}. Type: {type(error)}'
            )

        return True



"""Auth click sub-command"""
# pylint: skip-file

import click
from click_help_colors import HelpColorsCommand

from yojenkins.__main__ import auth
from yojenkins.cli import cli_auth, cli_decorators
from yojenkins.cli.cli_utility import set_debug_log_level
from yojenkins.utility.utility import translate_kwargs


@auth.command(short_help='\tConfigure yojenkins authentication profile(s)',
              cls=HelpColorsCommand,
              help_options_custom_colors={'--token': 'black'})
@cli_decorators.debug
@click.option('--auth-file',
              type=click.Path(file_okay=True, dir_okay=False),
              required=False,
              is_flag=False,
              help='JSON file containing one or more authentication info')
def configure(debug, auth_file):
    """Configure yojenkins authentication profile(s)

    This commnad can be used manually via terminal prompts, or via a JSON
    file containing one or more authentication info using --auth-file.
    Sample format of a "--auth-file" file, with single profile setup:

    \b
    {
        "<PROFILE NAME>": {
            "active": <true|false>,
            "api_token": "<API TOKEN>",
            "jenkins_server_url": "<SERVER BASE URL>",
            "username": "<USERNAME ID>"
        }
    }

    EXAMPLES:

    \b
    - yojenkins auth verify
    - yojenkins auth verify --auth-file my_auths.json
    """
    set_debug_log_level(debug)
    cli_auth.configure(auth_file)


@auth.command(short_help='\tGenerate and/or add server API token')
@cli_decorators.debug
# @cli_decorators.profile
@click.option('--profile',
              type=str,
              required=False,
              is_flag=False,
              help='yojenkins profile name or profile as JSON text')
@click.option('--token', type=str, required=False, help='API token to add to specified --profile')
@click.option('--name', type=str, required=False, help='Name of the generated API token')
@click.option('--server-base-url', type=str, required=False, help='Server base URL address')
@click.option('--username', type=str, required=False, help='Account/User username')
@click.option('--password', type=str, required=False, help='Account/User password')
def token(debug, **kwargs):
    """Generate and/or add a server authentication API token

    EXAMPLES:

    \b
    - yojenkins auth token --profile default
    - yojenkins auth token --profile my-profile --token 115cfc27acFAKE3c3773eaeb087d2aFAKE
    - yojenkins auth token --server-base-url "http://yo.com" --username "yo" --password "cool"
    - yojenkins auth token --profile myProfile5 --name "My Shiny Token"
    """
    set_debug_log_level(debug)
    cli_auth.token(**translate_kwargs(kwargs))


@auth.command(short_help='\tShow the local credentials profiles')
@cli_decorators.debug
@cli_decorators.format_output
def show(debug, **kwargs):
    """Show the local credentials profiles"""
    set_debug_log_level(debug)
    cli_auth.show(**translate_kwargs(kwargs))


@auth.command(short_help='\tCheck if current credentials can authenticate')
@cli_decorators.debug
@cli_decorators.profile
def verify(debug, **kwargs):
    """Check if current credentials can authenticate with server"""
    set_debug_log_level(debug)
    del kwargs['token']
    cli_auth.verify(**translate_kwargs(kwargs))


@auth.command(short_help='\tShow current user information')
@cli_decorators.debug
@cli_decorators.format_output
@cli_decorators.profile
def user(debug, **kwargs):
    """Show current user information"""
    set_debug_log_level(debug)
    cli_auth.user(**translate_kwargs(kwargs))


# @auth.command(short_help='\tWipe all credentials for this device')
# @cli_decorators.debug
# def wipe(debug):
#     """Wipe all credentials for this device"""
#     set_debug_log_level(debug)
#     click.secho('TODO :-/', fg='yellow')



"""Main entry point for program"""
# pylint: skip-file

import logging
import sys

import click
from click_help_colors import HelpColorsGroup

from yojenkins import __version__
from yojenkins.cli import logger_setup
from yojenkins.cli.cli_utility import set_debug_log_level

logger = logging.getLogger()

if sys.version_info < (3, 7):
    click.secho('Your Python version ({}.{}) is not supported'.format(sys.version_info.major, sys.version_info.minor), fg='bright_red', bold=True)
    click.secho('Must have Python 3.7 or higher', fg='bright_red', bold=True)
    sys.exit(1)

##############################################################################


MAIN_HELP_TEXT = f"""
    \t\t\t \033[93m YOJENKINS (Version: {__version__}) \033[0m

    yojenkins is a flexible tool that is focused on interfacing with
    Jenkins server from the comfort of the beloved command line.
    This tool can also be used as a middleware utility, generating and
    passing Jenkins information or automating tasks.

    QUICK START:

    \b
      1. Configure yo profile:  yojenkins auth configure
      2. Add yo API token:      yojenkins auth token --profile <PROFILE NAME>
      3. Verify yo creds:       yojenkins auth verify
      4. Explore yojenkins
"""


##############################################################################


@click.group(help=MAIN_HELP_TEXT)
@click.version_option(
    __version__, "-v", "--version", message="%(version)s".format(version="version"),
    help="Show the version"
)
def main():
    pass

# -----------------------------------------------------------------------------
@main.group(short_help='\tManage authentication and profiles',
    cls=HelpColorsGroup,
    help_options_custom_colors={
        'wipe': 'black'
        })
def auth():
    """Authentication And Profile Management"""
    pass
from yojenkins.cli_sub_commands import auth


# -----------------------------------------------------------------------------
@main.group(short_help='\tManage server')
def server():
    """Server Management"""
    pass
from yojenkins.cli_sub_commands import server


# -----------------------------------------------------------------------------
@main.group(short_help='\tManage nodes',
    cls=HelpColorsGroup,
    help_options_custom_colors={
        'prepare': 'black',
        'status': 'black',
        'create-ephemeral': 'black',
        'logs': 'black',
        })
def node():
    """Node Management"""
    pass
from yojenkins.cli_sub_commands import node


# -----------------------------------------------------------------------------
@main.group(short_help='\tManage user accounts',
cls=HelpColorsGroup,
    help_options_custom_colors={
        'password-reset': 'black',
        })
def account():
    """Account/User Management"""
    pass
from yojenkins.cli_sub_commands import account


# -----------------------------------------------------------------------------
@main.group(short_help='\tManage credentials',
cls=HelpColorsGroup,
    help_options_custom_colors={
        'update': 'black',
        'move': 'black'
        })
def credential():
    """Credentials Management"""
    pass
from yojenkins.cli_sub_commands import credential


# -----------------------------------------------------------------------------
@main.group(short_help='\tManage folders')
def folder():
    """Folder Management"""
    pass
from yojenkins.cli_sub_commands import folder


# -----------------------------------------------------------------------------
@main.group(short_help='\tManage jobs',
    cls=HelpColorsGroup,
    help_options_custom_colors={
        'queue_cancel': 'black'
        })
def job():
    """Job Management"""
    pass
from yojenkins.cli_sub_commands import job


# -----------------------------------------------------------------------------
@main.group(short_help='\tManage builds')
def build():
    """Build Management"""
    pass
from yojenkins.cli_sub_commands import build


# -----------------------------------------------------------------------------
@main.group(short_help='\tManage build stages')
def stage():
    """Stage Management"""
    pass
from yojenkins.cli_sub_commands import stage


# -----------------------------------------------------------------------------
@main.group(short_help='\tManage stage steps')
def step():
    """Step Management"""
    pass
from yojenkins.cli_sub_commands import step


# -----------------------------------------------------------------------------
@main.group(short_help='\tTools and more')
def tools():
    """Utility And More"""
    pass
from yojenkins.cli_sub_commands import tools

##############################################################################
##############################################################################
##############################################################################
if __name__ == "__main__":
    """Main entry point to the entire program"""
    main()



"""Job Menu CLI Entrypoints"""

import json
import logging
import sys

import click
import xmltodict

from yojenkins.cli import cli_utility as cu
from yojenkins.cli.cli_utility import log_to_history
from yojenkins.utility.utility import print2, wait_for_build_and_follow_logs

# Getting the logger reference
logger = logging.getLogger()


@log_to_history
def info(profile: str, token: str, job: str, **kwargs) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        data = yj_obj.job.info(job_url=job)
    else:
        data = yj_obj.job.info(job_name=job)
    cu.standard_out(data, **kwargs)


@log_to_history
def search(profile: str, token: str, search_pattern: str, search_folder: str, depth: int, fullname: bool,
           opt_list: bool, **kwargs) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(search_folder):
        data, data_list = yj_obj.job.search(search_pattern=search_pattern,
                                            folder_url=search_folder,
                                            folder_depth=depth,
                                            fullname=fullname)
    else:
        data, data_list = yj_obj.job.search(search_pattern=search_pattern,
                                            folder_name=search_folder,
                                            folder_depth=depth,
                                            fullname=fullname)
    if not data:
        print2("No folders found", color="yellow")
        sys.exit(1)
    data = data_list if opt_list else data
    cu.standard_out(data, **kwargs)


@log_to_history
def build_list(profile: str, token: str, opt_list: bool, job: str, **kwargs) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        data, data_list = yj_obj.job.build_list(job_url=job)
    else:
        data, data_list = yj_obj.job.build_list(job_name=job)
    data = data_list if opt_list else data
    cu.standard_out(data, **kwargs)


@log_to_history
def build_next(profile: str, token: str, job: str) -> None:
    """Get last build number for a job

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        data = yj_obj.job.build_next_number(job_url=job)
    else:
        data = yj_obj.job.build_next_number(job_name=job)
    click.secho(f'{data}', fg='bright_green', bold=True)


@log_to_history
def build_last(profile: str, token: str, job: str) -> None:
    """Get previous build number for a job

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        data = yj_obj.job.build_last_number(job_url=job)
    else:
        data = yj_obj.job.build_last_number(job_name=job)
    click.secho(f'{data}', fg='bright_green', bold=True)


@log_to_history
def build_set(profile: str, token: str, job: str, build_number: int) -> None:
    """Set the current build number for a job

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        yj_obj.job.build_set_next_number(build_number=build_number, job_url=job)
    else:
        yj_obj.job.build_set_next_number(build_number=build_number, job_name=job)
    click.secho(f'{build_number}', fg='bright_green', bold=True)


@log_to_history
def build_exist(profile: str, token: str, job: str, build_number: int) -> None:
    """Check if a build number for this job exists

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        data = yj_obj.job.build_number_exist(build_number, {}, job_url=job)
    else:
        data = yj_obj.job.build_number_exist(build_number, {}, job_name=job)
    if data:
        click.secho('true', fg='bright_green', bold=True)
    else:
        click.secho('false', fg='bright_red', bold=True)


@log_to_history
def build(profile: str, token: str, job: str, parameter: tuple, follow_logs: bool) -> None:
    """Build a job

    Args:
        profile:     The profile/account to use
        token:       API Token for Jenkins server
        job:         The job name under which the build is located
        parameter:   Specify key-value parameter. Can use multiple times. Use once per parameter
        follow_logs: Waits for the job build, then follows resulting logs
    """
    yj_obj = cu.config_yo_jenkins(profile, token)

    # Convert a tuple of tuples to dict
    parameters = dict(list(parameter))

    if cu.is_full_url(job):
        data = yj_obj.job.build_trigger(job_url=job, paramters=parameters)
    else:
        data = yj_obj.job.build_trigger(job_name=job, paramters=parameters)
    if not follow_logs:
        click.secho(f'success. queue number: {data}', fg='bright_green', bold=True)
        return
    wait_for_build_and_follow_logs(yj_obj, data)


@log_to_history
def queue_check(profile: str, token: str, job: str, opt_id: bool, **kwargs) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        data, queue_id = yj_obj.job.in_queue_check(job_url=job)
    else:
        data, queue_id = yj_obj.job.in_queue_check(job_name=job)
    if opt_id:
        click.secho(f'{queue_id}', fg='bright_green', bold=True)
    else:
        cu.standard_out(data, **kwargs)


@log_to_history
def browser(profile: str, token: str, job: str) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        yj_obj.job.browser_open(job_url=job)
    else:
        yj_obj.job.browser_open(job_name=job)


@log_to_history
def config(profile: str, token: str, opt_pretty: bool, opt_yaml: bool, opt_xml: bool, opt_toml: bool, opt_json: bool,
           job: str, filepath: str) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        data = yj_obj.job.config(filepath=filepath,
                                 job_url=job,
                                 opt_json=opt_json,
                                 opt_yaml=opt_yaml,
                                 opt_toml=opt_toml)
    else:
        data = yj_obj.job.config(filepath=filepath,
                                 job_name=job,
                                 opt_json=opt_json,
                                 opt_yaml=opt_yaml,
                                 opt_toml=opt_toml)

    # data = json.loads(json.dumps(xmltodict.parse(data)))  # Converting XML to dict
    opt_xml = not any([opt_json, opt_yaml, opt_toml])
    data = data if opt_xml else json.loads(json.dumps(xmltodict.parse(data)))
    cu.standard_out(data, opt_pretty, opt_yaml, opt_xml, opt_toml)


@log_to_history
def disable(profile: str, token: str, job: str) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        yj_obj.job.disable(job_url=job)
    else:
        yj_obj.job.disable(job_name=job)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def enable(profile: str, token: str, job: str) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        yj_obj.job.enable(job_url=job)
    else:
        yj_obj.job.enable(job_name=job)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def rename(profile: str, token: str, job: str, name: str) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        yj_obj.job.rename(new_name=name, job_url=job)
    else:
        yj_obj.job.rename(new_name=name, job_name=job)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def delete(profile: str, token: str, job: str) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        yj_obj.job.delete(job_url=job)
    else:
        yj_obj.job.delete(job_name=job)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def wipe(profile: str, token: str, job: str) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        yj_obj.job.wipe_workspace(job_url=job)
    else:
        yj_obj.job.wipe_workspace(job_name=job)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def monitor(profile: str, token: str, job: str, sound: bool) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        yj_obj.job.monitor(job_url=job, sound=sound)
    else:
        yj_obj.job.monitor(job_name=job, sound=sound)


@log_to_history
def create(profile: str, token: str, name: str, folder: str, config_file: str, config_is_json: bool) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(folder):
        yj_obj.job.create(name=name, folder_url=folder, config_file=config_file, config_is_json=config_is_json)
    else:
        yj_obj.job.create(name=name, folder_name=folder, config_file=config_file, config_is_json=config_is_json)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def parameters(profile: str, token: str, job: str, opt_list: bool, **kwargs) -> None:
    """TODO Docstring

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if cu.is_full_url(job):
        data, data_list = yj_obj.job.parameters(job_url=job)
    else:
        data, data_list = yj_obj.job.parameters(job_name=job)
    data = data_list if opt_list else data
    cu.standard_out(data, **kwargs)


@log_to_history
def diff(profile: str, token: str, job_1: str, job_2: str, no_color: bool, diff_only: bool, diff_guide: bool) -> None:
    """Get the diff comparison for two jobs

    Args:
        profile:     The profile/account to use
        token:       API Token for Jenkins server
        job_1:       First job for comparison (name or url)
        job_2:       Second job for comparison (name or url)
        no_color:    Output diff with no color
        diff_only:   Only show the lines that have changed
        diff_guide:  Show diff guide, showing where exactly difference is in line
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    yj_obj.job.diff(job_1, job_2, no_color, diff_only, diff_guide)



"""Click CLI Decorator Shortcuts/Utilities"""

import functools
from typing import Callable

import click


def format_output(decorated_function: Callable) -> Callable:
    """click module options for formatting the output

    Details: This function is a convenience function to use to add click options

    Args:
        decorated_function : Function that is decorated

    Returns:
        Decorated function
    """

    @click.option('-p',
                  '--pretty',
                  type=bool,
                  default=False,
                  required=False,
                  is_flag=True,
                  help='Output in pretty human readable format')
    @click.option('-y', '--yaml', type=bool, default=False, required=False, is_flag=True, help='Output in YAML format')
    @click.option('-x', '--xml', type=bool, default=False, required=False, is_flag=True, help='Output in XML format')
    @click.option('-t', '--toml', type=bool, default=False, required=False, is_flag=True, help='Output in TOML format')
    @functools.wraps(decorated_function)
    def wrapper(*args, **kwds):
        return decorated_function(*args, **kwds)

    return wrapper


def debug(decorated_function: Callable) -> Callable:
    """click module options for debug level

    Details: This function is a convenience function to use to add click options

    Args:
        decorated_function : Function that is decorated

    Returns:
        Decorated function
    """

    # TODO: Just call/move the debug log level update function here

    @click.option('--debug',
                  type=bool,
                  default=False,
                  required=False,
                  is_flag=True,
                  help='Enable debug level log messages')
    @functools.wraps(decorated_function)
    def wrapper(*args, **kwargs):
        return decorated_function(*args, **kwargs)

    return wrapper


def profile(decorated_function: Callable) -> Callable:
    """click module options for selecting the credentials profile to be used with the command

    Details: This function is a convenience function to use to add click options

    Args:
        decorated_function : Function that is decorated

    Returns:
        Decorated function
    """

    @click.option('--profile',
                  type=str,
                  required=False,
                  is_flag=False,
                  help='yojenkins profile name or profile as JSON text')
    @click.option('--token',
                  type=str,
                  required=False,
                  is_flag=False,
                  envvar="YOJENKINS_TOKEN",
                  help='Authentication API token to use')
    @functools.wraps(decorated_function)
    def wrapper(*args, **kwargs):
        return decorated_function(*args, **kwargs)

    return wrapper


def list(decorated_function: Callable) -> Callable:
    """click module options for outputting as a list

    Details: This function is a convenience function to use to add click options

    Args:
        decorated_function : Function that is decorated

    Returns:
        Decorated function
    """

    @click.option('-l', '--list', type=bool, default=False, required=False, is_flag=True, help='Output as list')
    @functools.wraps(decorated_function)
    def wrapper(*args, **kwargs):
        return decorated_function(*args, **kwargs)

    return wrapper



"""Server Menu CLI Entrypoints"""

import json
import logging
import os
from pathlib import Path

import click
from yaspin import yaspin
from yaspin.spinners import Spinners

from yojenkins.cli import cli_utility as cu
from yojenkins.cli.cli_utility import log_to_history
from yojenkins.docker_container import DockerJenkinsServer
from yojenkins.utility.utility import fail_out, failures_out, print2
from yojenkins.yo_jenkins import Auth, YoJenkins

# Getting the logger reference
logger = logging.getLogger()

# TODO: Find centralized location for these static values
CONFIG_DIR_NAME = '.yojenkins'


@log_to_history
def info(profile: str, token: str, **kwargs) -> None:
    """Get the server information

    Details: Targeting the server that is specified in the selected profile
    """
    data = cu.config_yo_jenkins(profile, token).server.info()
    cu.standard_out(data, **kwargs)


@log_to_history
def people(profile: str, token: str, opt_list: bool, **kwargs) -> None:
    """TODO Docstring

    Details: TODO

    Args:
        TODO
    """
    data, data_list = cu.config_yo_jenkins(profile, token).server.people()
    data = data_list if opt_list else data
    cu.standard_out(data, **kwargs)


@log_to_history
def queue(profile: str, token: str, opt_list: bool, **kwargs) -> None:
    """TODO Docstring

    Details: TODO

    Args:
        TODO
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    if opt_list:
        data = yj_obj.server.queue_list()  # TODO: Combine with server_queue_list adding a list argument
    else:
        data = yj_obj.server.queue_info()
    cu.standard_out(data, **kwargs)


@log_to_history
def plugins(profile: str, token: str, opt_list: bool, **kwargs) -> None:
    """TODO Docstring

    Details: TODO

    Args:
        TODO
    """
    data, data_list = cu.config_yo_jenkins(profile, token).server.plugin_list()
    data = data_list if opt_list else data
    cu.standard_out(data, **kwargs)


@log_to_history
def browser(profile: str, token: str) -> None:
    """TODO Docstring

    Details: TODO

    Args:
        TODO
    """
    cu.config_yo_jenkins(profile, token).server.browser_open()


@log_to_history
def reachable(profile: str, timeout: int) -> None:
    """TODO Docstring

    Details: TODO

    Args:
        TODO
    """
    auth = Auth()
    auth.get_credentials(profile)
    data = YoJenkins(auth=auth).rest.is_reachable(auth.jenkins_profile['jenkins_server_url'], timeout=timeout)
    if data:
        click.secho('true', fg='bright_green', bold=True)
    else:
        click.secho('false', fg='bright_red', bold=True)


@log_to_history
def quiet(profile: str, token: str, off: bool) -> None:
    """TODO Docstring

    Details: TODO

    Args:
        TODO
    """
    cu.config_yo_jenkins(profile, token).server.quiet(off=off)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def restart(profile: str, token: str, force: bool) -> None:
    """TODO Docstring

    Details: TODO

    Args:
        TODO
    """
    cu.config_yo_jenkins(profile, token).server.restart(force=force)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def shutdown(profile: str, token: str, force: bool) -> None:
    """TODO Docstring

    Details: TODO

    Args:
        TODO
    """
    cu.config_yo_jenkins(profile, token).server.shutdown(force=force)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def server_deploy(config_file: str, plugins_file: str, protocol_schema: str, host: str, port: int, image_base: str,
                  extra_setup_script: str, image_rebuild: bool, new_volume: bool, new_volume_name: str,
                  bind_mount_dir: str, container_name: str, registry: str, admin_user: str, password: str) -> None:
    """TODO Docstring

    Details: TODO

    Args:
        TODO
    """
    msg = "Setting up a local Jenkins development server. Hold tight, this may take a minute ..."
    if logger.level > 10:
        spinner = yaspin(spinner=getattr(Spinners, "bouncingBar"), attrs=["bold"], text=msg)
        spinner.start()
    else:
        click.echo(msg)

    # TODO: Check if the docker server deployment file is there. If so, show that it is being renewed.

    # Creat object
    djs = DockerJenkinsServer(config_file=config_file,
                              plugins_file=plugins_file,
                              protocol_schema=protocol_schema,
                              host=host,
                              port=port,
                              image_base=image_base,
                              extra_setup_script=extra_setup_script,
                              image_rebuild=image_rebuild,
                              new_volume=new_volume,
                              new_volume_name=new_volume_name,
                              bind_mount_dir=bind_mount_dir,
                              container_name=container_name,
                              registry=registry,
                              admin_user=admin_user,
                              password=password if password else "password")

    # Initialize docker client
    if not djs.docker_client_init():
        if logger.level > 10:
            spinner.stop()
        fail_out('Failed to connect to local docker client')

    # Setup the server
    deployed, success = djs.setup()

    if logger.level > 10:
        spinner.stop()

    if not success:
        failures = ['Failed to setup containerized server']
        if deployed:
            failures.append('Items deployed for partial deployment:')
            failures.append(deployed)
        failures_out(failures)

    # Write current server docker attributes to file
    filepath = os.path.join(Path.home(), CONFIG_DIR_NAME, 'last_deploy_info.json')
    if not filepath:
        fail_out('Failed to find yojenkins module included data directory')
    logger.debug(f'Writing server deploy information to file: {filepath}')
    try:
        with open(os.path.join(filepath), 'w') as file:
            json.dump(deployed, file, indent=4, sort_keys=True)
    except (IOError, PermissionError) as error:
        failures = [f'Server deployed, however failed to write server deploy information to file: {error}']
        failures.append('yojenkins resources may have been installed under root or a restricted account')
        failures_out(failures)

    volumes_named = [list(l.values())[0] for l in deployed["volumes"]]

    print2('Successfully created containerized Jenkins server!', bold=True, color='green')
    print2(f'   - Docker image:      {deployed["image"]}', bold=True, color='green')
    print2(f'   - Docker volumes:    {deployed["container"]}', bold=True, color='green')
    print2(f'   - Docker container:  {", ".join(volumes_named)}', bold=True, color='green')
    print2(f'   - Deployment file:   {filepath}', bold=True, color='green')
    click.echo()
    print2(f'Address:  {deployed["address"]}', bold=True, color='green')
    print2(f'Username: {admin_user}', bold=True, color='green')
    print2(f'Password: {"*************" if password else "password (default)"}', bold=True, color='green')


@log_to_history
def server_teardown(remove_volume: bool, remove_image: bool) -> None:
    """TODO Docstring

    Details: TODO

    Args:
        TODO
    """
    # Load deployment info file with deployment information
    filepath = os.path.join(Path.home(), CONFIG_DIR_NAME, 'last_deploy_info.json')
    logger.debug(f'Detecting server deployment info file: {filepath} ...')
    deployed = {}
    try:
        with open(os.path.join(filepath), 'r') as file:
            deployed = json.load(file)
        logger.debug(f'Successfully found and loaded server deployment info file: {deployed}')
    except (FileNotFoundError, FileExistsError):
        failures = ['Failed to find server deployment info file']
        failures.append('If you think there was a previously successfull deployment, use Docker to remove it manually')
        failures_out(failures)
    except (IOError, PermissionError) as error:
        fail_out(f'Failed to load previous server deployment info file: {error}')

    # Filter out named volumes only
    volumes_named_only = [list(l.values())[0] for l in deployed["volumes"] if 'named' in l]

    # Create object
    djs = DockerJenkinsServer(image_fullname=deployed['image'],
                              new_volume_name=volumes_named_only[0],
                              container_name=deployed['container'])

    # Initialize docker client
    if not djs.docker_client_init():
        print2('Failed to connect to local docker client')

    # Remove the resources
    if not djs.clean(remove_volume, remove_image):
        fail_out('Failed to remove Jenkins server')

    # Remove the deployment info file
    os.remove(filepath)

    print2(f'Successfully removed Jenkins server: {deployed["address"]}', bold=True, color='green')



"""Build Menu CLI Entrypoints"""

import logging
import sys

import click

from yojenkins.cli import cli_utility as cu
from yojenkins.cli.cli_utility import log_to_history
from yojenkins.utility.utility import is_complete_build_url, wait_for_build_and_follow_logs
from yojenkins.yo_jenkins.status import Status

# Getting the logger reference
logger = logging.getLogger()


def _verify_build_url_get_job_format(build_url: str, job: str) -> bool:
    """Utility function to verify build url and determine if job is URL or name

    Args:
        build_url: Build URL
        job: Job URL or name

    Returns:
         True if both are valid, False otherwise
    """
    if build_url:
        if not cu.is_full_url(build_url):
            click.secho('The build url not formatted correctly', fg='bright_red', bold=True)
            sys.exit(1)
        valid_url_format = True
    else:
        valid_url_format = cu.is_full_url(job)
    return valid_url_format


@log_to_history
def info(profile: str, token: str, job: str, number: int, url: str, latest: bool, **kwargs) -> None:
    """Fetching build information

    Args:
        profile: The profile/account to use
        token:   API Token for Jenkins server
        job:     The job this build is under
        number:  The build number to get info on
        url:     The build url to get info on
        latest:  Option to get the latest build
    """
    if url is None and job and is_complete_build_url(job):
        url, job = job, None
    elif job and not number and not latest:
        click.echo(
            click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                        fg='bright_red',
                        bold=True))
        sys.exit(1)

    yj_obj = cu.config_yo_jenkins(profile, token)

    if _verify_build_url_get_job_format(build_url=url, job=job):
        data = yj_obj.build.info(build_url=url, job_url=job, build_number=number, latest=latest)
    else:
        data = yj_obj.build.info(build_url=url, job_name=job, build_number=number, latest=latest)
    cu.standard_out(data, **kwargs)


@log_to_history
def status(profile: str, token: str, job: str, number: int, url: str, latest: bool) -> None:
    """Build status text/label

    ### FIXME: Verify build number resolution works

    Args:
        profile: The profile/account to use
        token:   API Token for Jenkins server
        job: The job this build is under
        number: The build number to get info on
        url: The build url to get info on
        latest: Option to get the latest build
    """
    if url is None and job and is_complete_build_url(job):
        url, job = job, None
    elif job and not number and not latest:
        click.echo(
            click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                        fg='bright_red',
                        bold=True))
        sys.exit(1)

    yj_obj = cu.config_yo_jenkins(profile, token)

    if _verify_build_url_get_job_format(build_url=url, job=job):
        data = yj_obj.build.status_text(build_url=url, job_url=job, build_number=number, latest=latest)
    else:
        data = yj_obj.build.status_text(build_url=url, job_name=job, build_number=number, latest=latest)

    # Color for output
    if data in Status.NONE.value:
        data = "NONE"
        output_fg = 'black'
    elif data.upper() in Status.UNKNOWN.value:
        output_fg = 'black'
    elif data.upper() in Status.QUEUED.value:
        output_fg = 'yellow'
    elif data.upper() in Status.RUNNING.value:
        output_fg = 'blue'
    elif data.upper() in Status.SUCCESS.value:
        output_fg = 'bright_green'
    elif data.upper() in Status.FAILURE.value:
        output_fg = 'bright_red'
    else:
        output_fg = ''
    click.secho(f'{data}', fg=output_fg, bold=True)


@log_to_history
def abort(profile: str, token: str, job: str, number: int, url: str, latest: bool) -> None:
    """Abort build

    Args:
        profile: The profile/account to use
        token:   API Token for Jenkins server
        job: The job this build is under
        number: The build number to get info on
        url: The build url to get info on
        latest: Option to get the latest build
    """
    if url is None and job and is_complete_build_url(job):
        url, job = job, None
    elif job and not number and not latest:
        click.echo(
            click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                        fg='bright_red',
                        bold=True))
        sys.exit(1)

    yj_obj = cu.config_yo_jenkins(profile, token)

    if _verify_build_url_get_job_format(build_url=url, job=job):
        yj_obj.build.abort(build_url=url, job_url=job, build_number=number, latest=latest)
    else:
        yj_obj.build.abort(build_url=url, job_name=job, build_number=number, latest=latest)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def delete(profile: str, token: str, job: str, number: int, url: str, latest: bool) -> None:
    """Delete build

    Args:
        profile: The profile/account to use
        token:   API Token for Jenkins server
        job: The job this build is under
        number: The build number to get info on
        url: The build url to get info on
        latest: Option to get the latest build
    """
    if url is None and job and is_complete_build_url(job):
        url, job = job, None
    elif job and not number and not latest:
        click.echo(
            click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                        fg='bright_red',
                        bold=True))
        sys.exit(1)

    yj_obj = cu.config_yo_jenkins(profile, token)

    if _verify_build_url_get_job_format(build_url=url, job=job):
        yj_obj.build.delete(build_url=url, job_url=job, build_number=number, latest=latest)
    else:
        yj_obj.build.delete(build_url=url, job_name=job, build_number=number, latest=latest)
    click.secho('success', fg='bright_green', bold=True)


@log_to_history
def stages(profile: str, token: str, opt_list: bool, job: str, number: int, url: str, latest: bool, **kwargs) -> None:
    """Get build stages

    Args:
        profile: The profile/account to use
        token:   API Token for Jenkins server
        opt_list: Option to list all stages without details
        job: The job name under which the build is located
        number: The build number
        url: The build URL
        latest: Option to get the latest build
    """
    if url is None and job and is_complete_build_url(job):
        url, job = job, None
    elif job and not number and not latest:
        click.echo(
            click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                        fg='bright_red',
                        bold=True))
        sys.exit(1)

    yj_obj = cu.config_yo_jenkins(profile, token)

    if _verify_build_url_get_job_format(build_url=url, job=job):
        data, data_list = yj_obj.build.stage_list(build_url=url, job_url=job, build_number=number, latest=latest)
    else:
        data, data_list = yj_obj.build.stage_list(build_url=url, job_name=job, build_number=number, latest=latest)
    data = data_list if opt_list else data
    cu.standard_out(data, **kwargs)


@log_to_history
def logs(profile: str, token: str, job: str, number: int, url: str, latest: bool, tail: float, download_dir: str,
         follow: bool) -> None:
    """Get build logs

    Args:
        profile: The profile/account to use
        token:   API Token for Jenkins server
        job: The job this build is under
        number: The build number to get info on
        url: The build url to get info on
        latest: Option to get the latest build
        tail: Option to get the last N lines of the log
        download_dir: Option to download the log to a directory
        follow: Option to follow the log
    """
    if url is None and job and is_complete_build_url(job):
        url, job = job, None
    elif job and not number and not latest:
        click.echo(
            click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                        fg='bright_red',
                        bold=True))
        sys.exit(1)

    yj_obj = cu.config_yo_jenkins(profile, token)

    if _verify_build_url_get_job_format(build_url=url, job=job):
        yj_obj.build.logs(build_url=url,
                          job_url=job,
                          build_number=number,
                          latest=latest,
                          tail=tail,
                          download_dir=download_dir,
                          follow=follow)
    else:
        yj_obj.build.logs(build_url=url,
                          job_name=job,
                          build_number=number,
                          latest=latest,
                          tail=tail,
                          download_dir=download_dir,
                          follow=follow)
    if download_dir:
        click.secho('success', fg='bright_green', bold=True)


@log_to_history
def browser(profile: str, token: str, job: str, number: int, url: str, latest: bool) -> None:
    """Open build in web browser

    Args:
        profile: The profile/account to use
        token:   API Token for Jenkins server
        job: The job this build is under
        number: The build number to get info on
        url: The build url to get info on
        latest: Option to get the latest build
    """
    if url is None and job and is_complete_build_url(job):
        url, job = job, None
    elif job and not number and not latest:
        click.echo(
            click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                        fg='bright_red',
                        bold=True))
        sys.exit(1)

    yj_obj = cu.config_yo_jenkins(profile, token)

    if _verify_build_url_get_job_format(build_url=url, job=job):
        yj_obj.build.browser_open(build_url=url, job_url=job, build_number=number, latest=latest)
    else:
        yj_obj.build.browser_open(build_url=url, job_name=job, build_number=number, latest=latest)


@log_to_history
def monitor(profile: str, token: str, job: str, number: int, url: str, latest: bool, sound: bool) -> None:
    """Start monitor UI

    Args:
        profile: The profile/account to use
        token:   API Token for Jenkins server
        job: The job this build is under
        number: The build number to get info on
        url: The build url to get info on
        latest: Option to get the latest build
        sound: Option to play a sound when the build status changes
    """
    if url is None and job and is_complete_build_url(job):
        url, job = job, None
    elif job and not number and not latest:
        click.echo(
            click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                        fg='bright_red',
                        bold=True))
        sys.exit(1)

    yj_obj = cu.config_yo_jenkins(profile, token)

    if _verify_build_url_get_job_format(build_url=url, job=job):
        yj_obj.build.monitor(build_url=url, job_url=job, build_number=number, latest=latest, sound=sound)
    else:
        yj_obj.build.monitor(build_url=url, job_name=job, build_number=number, latest=latest, sound=sound)


@log_to_history
def parameters(profile: str, token: str, opt_list: bool, job: str, number: int, url: str, latest: bool,
               **kwargs) -> None:
    """Get build parameters

    Args:
        profile: The profile/account to use
        token:   API Token for Jenkins server
        opt_list: Option to list all stages without details
        job: The job name under which the build is located
        number: The build number
        url: The build URL
        latest: Option to get the latest build
    """
    if url is None and job and is_complete_build_url(job):
        url, job = job, None
    elif job and not number and not latest:
        click.echo(
            click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                        fg='bright_red',
                        bold=True))
        sys.exit(1)

    yj_obj = cu.config_yo_jenkins(profile, token)

    if _verify_build_url_get_job_format(build_url=url, job=job):
        data, data_list = yj_obj.build.parameters(build_url=url, job_url=job, build_number=number, latest=latest)
    else:
        data, data_list = yj_obj.build.parameters(build_url=url, job_name=job, build_number=number, latest=latest)
    data = data_list if opt_list else data
    cu.standard_out(data, **kwargs)


@log_to_history
def rebuild(profile: str, token: str, job: str, number: int, url: str, latest: bool, follow_logs: bool) -> None:
    """Rebuild a build with same setup/parameters

    Args:
        profile: The profile/account to use
        token:   API Token for Jenkins server
        job:     The job name under which the build is located
        number:  The build number
        url:     The build URL
        latest:  Option to get the latest build
        follow_logs: Waits for the job build, then follows resulting logs
    """

    if url is None and job and is_complete_build_url(job):
        url, job = job, None
    elif job and not number and not latest:
        click.echo(
            click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                        fg='bright_red',
                        bold=True))
        sys.exit(1)

    yj_obj = cu.config_yo_jenkins(profile, token)

    if _verify_build_url_get_job_format(build_url=url, job=job):
        data = yj_obj.build.rebuild(build_url=url, job_url=job, build_number=number, latest=latest)
    else:
        data = yj_obj.build.rebuild(build_url=url, job_name=job, build_number=number, latest=latest)

    if not follow_logs:
        click.secho(f'success. queue number: {data}', fg='bright_green', bold=True)
        return
    wait_for_build_and_follow_logs(yj_obj, data)


@log_to_history
def diff(profile: str, token: str, build_url_1: str, build_url_2: str, logs: bool, line_pattern: tuple,
         char_ignore: int, no_color: bool, diff_only: bool, diff_guide: bool) -> None:
    """Get the diff comparison for two builds

    Args:
        profile:      The profile/account to use
        token:        API Token for Jenkins server
        build_url_1:  First build for comparison
        build_url_2:  Second build for comparison
        logs:         Compare build logs
        line-pattern: Patterns to consider for diff for each line
        char_ignore:  Number of characters to ignore at start of each line
        no_color:     Output diff with no color
        diff_only:    Only show the lines that have changed
        diff_guide:   Show diff guide, showing where exactly difference is in line
    """
    yj_obj = cu.config_yo_jenkins(profile, token)
    yj_obj.build.diff(build_url_1, build_url_2, logs, line_pattern, char_ignore, no_color, diff_only, diff_guide)



"""Folder monitor"""

# import curses
import logging

from yojenkins.monitor.monitor import Monitor

# Getting the logger reference
logger = logging.getLogger()


class FolderMonitor(Monitor):
    """This class defines the BuildMonitor class and its function.

    The FolderMonitor class enables active folder monitoring
    """

    def __init__(self, Build) -> None:
        """Object constructor method, called at object creation

        Args:
            Build: Build object

        Returns:
            None
        """
        # Get attributes from super (parent) class
        super().__init__()

        pass
